@@ -1,8 +1,8 @@
 from .core import (groupby, remove, concat, concatv, mapcat,
                    frequencies, interpose, first, second, nth, take,
                    drop, rest, get, last, merge_sorted, interleave,
-                   unique, reduceby, intersection, isiterable,
+                   unique, reduceby, isiterable,
                    isdistinct, cons, iterate, accumulate, sliding_window,
-                   count, partition, partition_all)
+                   count, partition, partition_all, take_nth)
 
 from .recipes import countby, partitionby
